@@ -18,6 +18,7 @@ from http.cookies import BaseCookie
 from wsgiref.util import setup_testing_defaults
 from wsgiref.validate import validator
 import io
+import time
 
 import pytest
 
@@ -26,6 +27,7 @@ from obsession import (
     CookieIdPersistence,
     Session,
     MemoryBackend,
+    COOKIE_NAME,
 )
 from obsession import JSONSerializer
 
@@ -146,7 +148,7 @@ class TestSessionMiddleware(object):
         a = SessionMiddleware(app(deletes_the_session), backend=backend)
         r = Response(a, HTTP_COOKIE="_obs=" + r.cookies["_obs"].coded_value)
         assert len(backend.keys()) == 0
-        assert len(r.cookies) == 0
+        assert "Max-Age=-1" in dict(r.headers)["Set-Cookie"]
 
     def test_sessions_are_independent(self):
         a = SessionMiddleware(app(increments_a_session_value))
@@ -188,16 +190,16 @@ class TestSession(object):
     def test_it_implements_mutable_mapping_abstract_methods(self):
         s = Session({}, MemoryBackend(), lambda e: "")
         with pytest.raises(KeyError):
-            s['foo']
+            s["foo"]
         with pytest.raises(KeyError):
-            del s['foo']
+            del s["foo"]
         assert list(s) == []
         assert len(s) == 0
-        s['foo'] = 1
-        assert s['foo'] == 1
-        assert list(s) == ['foo']
+        s["foo"] = 1
+        assert s["foo"] == 1
+        assert list(s) == ["foo"]
         assert len(s) == 1
-        del s['foo']
+        del s["foo"]
         assert len(s) == 0
 
 
@@ -243,6 +245,35 @@ class TestCookieIdPersistence(object):
         id = ".12345678901234567890123456789012345678."
         assert cip.get_id({"HTTP_COOKIE": "_obs=" + id}) == id
 
+    def test_it_reissues_cookie_close_to_expiry(self):
+        backend = MemoryBackend()
+        start_time = time.mktime(time.localtime())
+
+        def getapp(**kw):
+            return SessionMiddleware(
+                app(sets_a_session_key),
+                backend=backend,
+                id_persister=CookieIdPersistence(max_age=60, **kw),
+            )
+
+        r = Response(getapp())
+        id = r.cookies[COOKIE_NAME].value
+
+        # After 2s - no new cookie
+        r = Response(
+            getapp(_localtime=lambda: time.localtime(start_time + 2)),
+            {"HTTP_COOKIE": f"{COOKIE_NAME}={id}"},
+        )
+        assert len(r.cookies) == 0
+
+        # After 40s - resend cookie to reset expiry
+        r = Response(
+            getapp(_localtime=lambda: time.localtime(start_time + 40)),
+            {"HTTP_COOKIE": f"{COOKIE_NAME}={id}"},
+        )
+        assert len(r.cookies) == 1
+        assert r.cookies[COOKIE_NAME].value == id
+
 
 class TestMemoryBackend(object):
     def test_it_expires_old_items(self):
